// backseatSiege - a QuorraBot plugin.
// Created by by vitastic (on Twitch) / worstje (on DonationCoder.com)
// Released for free as a part of the NANY 2017 celebration on DonationCoder.
//
// Legal disclaimer (aka how free is this script?):
//   As long as you leave all references to the original author (be it website,
//   name or otherwise) intact, you are free to use this plugin as you see fit.
//   I am not giving any warranty, nor am I taking on any liability.
//   This script is provided AS-IS.
//
// TODO list:
//   * Aliases (configurable!) for the different operators.
//     Capitão & Jäger are really in need of that in particular.
//     Although 'BB' for blackbeard or 'cav' for caveira would also be nice.
//   * Maybe at some point do a less bare-bones 'stream' interface than simply
//     copying images around like I do now.
//
// Version history:
//   1.0 (summer 2016)
//     Initial version for Quorrabot v1.12.
//   1.1 (2016-12-28)
//     Added tampering and additional debugging commands.
//   1.2 (2017-01-02)
//     Cleaned up source code for DonationCoder release.
//     Made sure to finally add Hibana & Echo (DLC Season #4 operators.)
//     Added installation instructions & changelog.
//     Verified functional for Quorrabot v1.14.
//     Added barebones legal stuff. Just in case.
//     Happy New Year! Let's hope 2017 will be a smaller disaster than 2016!
//
// Installation instructions:
//   0. Install & configure Quorrabot. (Duh.)
//   1. Put backseatSiegeSystem.js in the scripts/systems subdirectory.
//   2. Configure backseatSiegeSystem.js in the area marked 'CONFIGURATION'
//      somewhere below all this text.
//   3. Prepare the directory with all the game badges as you've configured.
//      You can likely download these one by one from the official game website.
//      (I am unsure if I can offer a download for these for the obvious legal
//       reasons, so at present I have to refer you to the slower manual way.)
//   4. Enable the backseatSiegeSystem module. The command ought to be:
//        !module enable ./systems/backseatSiegeSystem.js
//      (Note: This path appears to be relative to the scripts subdirectory.)
//   5. Give the system a quick test run by activating it and placing some votes.
//      The easiest way to do so ought to be to type this in your Twitch chat:
//         !backseat on
//         !vote doc iq
//         !went bandit
//   6. Now prepare your streaming software so that it uses the configured
//      output images. (See the BadgeLeader* and BadgeRightNow settings.)
//      You will want to configure your software to automatically detect
//      changes to these files. If all is well, your current attacker ought to
//      display as BANDIT, the leading audience vote for DEF ought to be DOC
//      and for the ATK side it ought to be IQ.
//   7. Also, use something to open your VoteRankingsFile that can detect and
//      auto-load the changes. You ought to see lots of zeroes, and two single
//      votes on the top row for both DOC and IQ.
//   8. Now the real test to make sure all your stuff is properly set up:
//        !backseat tamper all 22
//      This sets the votes for all operators to 22. (You can also use 0,
//      but I figured this is less boring to look at in the vote rankings!)
//      On your stream, you ought to see the audience choice leaders cycle
//      through all the operators, once every second or so. In the text file,
//      you will see an arrow moving down the list every second to match what
//      is currently visible on your stream as the leader.
//   9. Enjoy? (Also, turn off the system to stop the updates! !backseat off)
//
// Usage instructions:
//   1. !backseat on      (also make sure to show the proper stuff on-stream)
//                        (also open the vote rankings so you can act on them!)
//   2. !went XXX         (after choosing an operator; this gives people the
//                         ability to vote again for that faction.)
//   3. Encourage people to use !vote ...
//   4. !backseat off     (once you are done with all the backseat gaming...)
//
//
// Script written by vitastic in order to facilitate
// the audience for determining the operators played
// by the streamer in the game Rainbow Six Siege.
//
// Intended features:
//   * Separate per-viewer last-voted timers for ATK/DEF
//   * Copies the currently leading ATK/DEF operators to
//     another file for leaderboard displaying purposes.
//   * Votes for operators that did not make the cut will
//     linger and still be there to give those less-liked
//     operators a head-start over the more popular ones.
//   * Resets viewer ability to vote based on the time of
//     said vote by comparing against streamers most
//     recent confirmation of an ingame selected operator.
//     Obviously, that only applies to the faction in
//     question. :-)
//
//
// How to use:
//   * !play <operator> [<operator>]   (for viewers; anything else is ignored)
//     This command is how viewers vote for the next operator they wish to see played.
//     Only one of each side (ATK/DEF) is allowed. One cannot vote for the same side
//     again until the streamer .went with that faction once.
//   * !backseat on|off|toggle         (for streamer)
//     This turns the voting capacity on/off.
//   * !went <operator>                (for streamer)
//     This selects the operator currently chosen.
//
// Debugging commands: (not part of intended usage pattern)
//   * !backseat reset atk|def|all     (for streamer)
//     This manually resets the ability for viewers to vote for the specified
//     faction(s). (It is basically !went without the operator clear!)
//   * !backseat tamper <operator> [+|-]<number>
//     Tamper with the votes for a particular operator (or all of them!) by
//     setting them to a specific value or giving them a point boost/handicap.

// CONFIGURATION STARTS HERE.

// NOTE: Paths are relative to Quorrabot executable file.
//       (They are for me, at least.)


$.backseatSiege_BadgesDirectory = "../siege-operator-badges/official/";
$.backseatSiege_BadgesExtension = ".png";

$.backseatSiege_BadgeLeaderATK  = "../audience-choice-ATK.png";
$.backseatSiege_BadgeLeaderDEF  = "../audience-choice-DEF.png";

$.backseatSiege_BadgeRightNow  = "../playing-right-now.png";

$.backseatSiege_VoteRankingsFile  = "../vote-rankings.txt";

// The script expects to find 'badges' with these names and above-configured
// BadgesExtension in the above-configured BadgesDirectory.
$.backseatSiege_OperatorFactions = {
	// FBI SWAT
	"ash"        : "atk",          "castle"    : "def",
	"thermite"   : "atk",	         "pulse"     : "def",
	// GIGN
	"montagne"   : "atk",          "doc"       : "def",
	"twitch"     : "atk",          "rook"      : "def",
	// GSG9
	"blitz"      : "atk",          "bandit"    : "def",
	"iq"         : "atk",          "jager"     : "def",
	// SAS
	"sledge"     : "atk",          "mute"      : "def",
	"thatcher"   : "atk",          "smoke"     : "def",
	// Spetsnaz
	"fuze"       : "atk",          "kapkan"    : "def",
	"glaz"       : "atk",          "tachanka"  : "def",
	// JTF2               (Season 1 DLC)
	"buck"       : "atk",          "frost"     : "def",
	// Navy SEAL          (Season 2 DLC)
	"blackbeard" : "atk",          "valkyrie"  : "def",
	// BOPE               (Season 3 DLC)
	"capitao"    : "atk",          "caveira"   : "def",
	// SAT                (Season 4 DLC)
	"hibana"     : "atk",          "echo"      : "def",
}
// CONFIGURATION ENDS HERE.

// Messages visible to users. (Translations / Personalization!)
$.lang.data["net.quorrabot.backseatSiegeSystem.not-enabled"] = "Backseat gaming? Nobody ain't got no time for that right now!";
$.lang.data["net.quorrabot.backseatSiegeSystem.play-what"] = "Play? Oh, he will do that. But which operator should he play?";
$.lang.data["net.quorrabot.backseatSiegeSystem.voting-enabled"] = "The polling booths are OPEN. Get in your votes: what operator will he !play next ?!";
$.lang.data["net.quorrabot.backseatSiegeSystem.voting-disabled"] = "The polling booths have CLOSED. Better luck next time!";
$.lang.data["net.quorrabot.backseatSiegeSystem.reset-what"] = "Please specify which votes to reset. (ATK, DEF, ALL)";
$.lang.data["net.quorrabot.backseatSiegeSystem.reset-done"] = "Done.";
$.lang.data["net.quorrabot.backseatSiegeSystem.tamper-invalid-target"] = "That is not a valid target to tamper with. (Valid options: any operator, ATK, DEF, ALL)"
$.lang.data["net.quorrabot.backseatSiegeSystem.tamper-invalid-tamper"] = "That is not a way one can tamper with things. (Valid options: X, +X, -X, where X is an integer)"
$.lang.data["net.quorrabot.backseatSiegeSystem.tamper-complete"] = "Tampering complete. Hurray, democracy!"

$.lang.data["net.quorrabot.backseatSiegeSystem.operator-chosen"] = "Operator $1 was chosen with a total of $2 votes.";
$.lang.data["net.quorrabot.backseatSiegeSystem.vote-again-atk"] = "Everybody can vote once more for an attacking operator.";
$.lang.data["net.quorrabot.backseatSiegeSystem.vote-again-def"] = "Everybody can vote once more for a defending operator.";
// $.lang.data["net.quorrabot.backseatSiegeSystem.vote-accepted"] = "Your vote for $1 has been registered!";
$.lang.data["net.quorrabot.backseatSiegeSystem.vote-spam-atk"] = "You cannot vote for $1 right now because you have already used your vote for an attacking operator.";
$.lang.data["net.quorrabot.backseatSiegeSystem.vote-spam-def"] = "You cannot vote for $1 right now because you have already used your vote for a defending operator.";

// Structure of database storage.

// backseatSiege_votes           [OPERATOR, VOTES]       -- stores audience votes for every operator
// backseatSiege_lastvote_atk    [USERNAME, TIME]        -- stores most recent audience vote time for attacker faction
// backseatSiege_lastvote_def    [USERNAME, TIME]        -- stores most recent audience vote time for defender faction
// backseatSiege_misc            [KEY, VALUE]            -- stores the most recent time the streamer !went ATK / DEF

// Javascript has crazy syntax I keep having to Google. And why calculate it more than once?
$.backseatSiege_OperatorCount = Object.keys(backseatSiege_OperatorFactions).length;

$.backseatInitializeBooleanHelper = function(v)
{
	  if (v == null)
	    return null;
	  else if ((v == true) || (v.toString().equalsIgnoreCase("true")))
	  	return true;
	  else
	  	return false;
}

// Internal variables.
$.backseatSiege_Toggle = $.backseatInitializeBooleanHelper($.inidb.get("settings", "backseatSiege_Toggle"));
if ($.backseatSiege_Toggle == null) {
    $.backseatSiege_Toggle = false;
    $.inidb.set("settings", "backseatSiege_Toggle", "false");
}

// Lazily loaded on a timer.
$.backseatSiege_VoteRankings = {};
$.backseatSiege_VoteRankingsATK = [];
$.backseatSiege_VoteRankingsDEF = [];

$.backseatSiege_lastBadgeATK = "";
$.backseatSiege_lastBadgeDEF = "";

// Cache and avoid excessive database hits when people vote.
$.backseatSiege_lastvote_atk = new Number($.inidb.get("backseatSiege_misc", "lastvote_atk"));
$.backseatSiege_lastvote_def = new Number($.inidb.get("backseatSiege_misc", "lastvote_def"));

// Internal functions.
$.backseatSiege_copyFile = function(file,path) {
    var file = new java.io.File(file);
    var path = new java.io.File(path);
    if((file!=null && path!=null)|| (file!="" && path!=""))
    {
        org.apache.commons.io.FileUtils.copyFile(file, path, false);
    }
}

$.backseatSiege_rotateTickerATK = $.randRange(0, Math.floor($.backseatSiege_OperatorCount/2) - 1);
$.backseatSiege_rotateTickerDEF = $.randRange(0, Math.floor($.backseatSiege_OperatorCount/2) - 1);

$.backseatSiege_rotateTicker = function(ticker)
{
	  return (ticker + 1) % ($.backseatSiege_OperatorCount/2);
}

$.backseatSiege_rotateLeader = function(operators, operatorVotes, ticker)
{
	  // Count the number of ties. This assumes a descending sorted list.
	  var ties = 0;
	  var highest = operatorVotes[operators[0]];
	  while ((operatorVotes[operators[ties]] == highest) && (ties < operators.length))
	  {
	  	  ties = ties + 1;
	  }

	  // Only a single tie? Then there is no tie! (Wow, that's deep.) And nothing to cycle.
	  if (ties == 1)
	  {
	  	  return "solo";
	  }
		return ticker % ties;
}

$.backseatSiege_updateRankings = function()
{
	  // Get all the recent rankings.
	  var operatorVotes = {}
	  var operatorsATK = [];
	  var operatorsDEF = [];

		for (var operator in $.backseatSiege_OperatorFactions) {
        operatorVotes[operator] = $.backseatSiege_GetOperatorVoteCount(operator);

        if ($.backseatSiege_OperatorFactions[operator] == "atk")
        {
        	  operatorsATK.push(operator);
        }
        if ($.backseatSiege_OperatorFactions[operator] == "def")
        {
        	  operatorsDEF.push(operator);
		    }
		}

		// Sort the two faction lists.
		var sortingFunc = function(a, b)
		{
        if(operatorVotes[a] > operatorVotes[b]) return -1;
        if(operatorVotes[a] < operatorVotes[b]) return 1;
        return 0;
    }

		operatorsATK.sort(sortingFunc);
		operatorsDEF.sort(sortingFunc);

		// Make sure the ticker cycles.
		var idxATK = $.backseatSiege_rotateLeader(operatorsATK, operatorVotes, $.backseatSiege_rotateTickerATK);
		if (idxATK != "solo")
		{
	      $.backseatSiege_rotateTickerATK = $.backseatSiege_rotateTicker($.backseatSiege_rotateTickerATK);
	  }
		else
		{
  			idxATK = 0;
		}
		var idxDEF = $.backseatSiege_rotateLeader(operatorsDEF, operatorVotes, $.backseatSiege_rotateTickerDEF);
		if (idxDEF != "solo")
		{
	      $.backseatSiege_rotateTickerDEF = $.backseatSiege_rotateTicker($.backseatSiege_rotateTickerDEF);
	  }
		else
		{
			  idxDEF = 0;
		}

		// Only copy over files when the lead of a faction changes to reduce wear & tear.
		if (operatorsATK[idxATK] != $.backseatSiege_VoteRankingsATK[$.backseatSiege_lastBadgeATK])
		{
  		  // Copy over new image.
			  var operator_file = backseatSiege_BadgesDirectory + operatorsATK[idxATK] + backseatSiege_BadgesExtension;
			  $.backseatSiege_copyFile(operator_file, backseatSiege_BadgeLeaderATK);
	  }
	  if (operatorsDEF[idxDEF] != $.backseatSiege_VoteRankingsDEF[$.backseatSiege_lastBadgeDEF])
		{
			  // Copy over new image.
			  var operator_file = backseatSiege_BadgesDirectory + operatorsDEF[idxDEF] + backseatSiege_BadgesExtension;
			  $.backseatSiege_copyFile(operator_file, backseatSiege_BadgeLeaderDEF);
	  }

		// In case we want to refer to it later...
		$.backseatSiege_VoteRankings = operatorVotes;
	  $.backseatSiege_VoteRankingsATK = operatorsATK;
		$.backseatSiege_VoteRankingsDEF = operatorsDEF;

		$.backseatSiege_lastBadgeATK = idxATK;
		$.backseatSiege_lastBadgeDEF = idxDEF;

		// Update text file with standings.
		$.backseatSiege_WriteVoteRankingsFile(operatorVotes, operatorsATK, operatorsDEF, idxATK, idxDEF);

		return;
}

$.backseatSiege_WriteVoteRankingsFile = function(operatorVotes, operatorsATK, operatorsDEF, idxATK, idxDEF)
{
    // Write a file that contains two columns with the standings.
		try {
				var fos = new java.io.FileOutputStream ($.backseatSiege_VoteRankingsFile, false);
				var ps = new java.io.PrintStream (fos);

				var currentColumn = 0;

				for (var i = 0; i < Math.floor($.backseatSiege_OperatorCount/2); i++)
				{
		        // Output them one by one. ATK column followed by DEF column.
						currentColumn = 0;
						var prettyATK = backseatSiege_CapitalizeFirstLetter(operatorsATK[i]);
						var prettyDEF = backseatSiege_CapitalizeFirstLetter(operatorsDEF[i]);

						// If badge spinner is currently on this item, make it completely uppercase.
						if (i == idxATK)
						{
								prettyATK = "> " + prettyATK.toUpperCase();
						}
						else
						{
							  prettyATK = "| " + prettyATK;
						}

						if (i == idxDEF)
						{
								prettyDEF = "> " + prettyDEF.toUpperCase();
						}
						else
						{
						  	prettyDEF = "| " + prettyDEF;
						}

						// ps.println (string);
						//    324. --> BUCK          <--
						ps.format("%7.0f %-12s    %7.0f %s \r\n",
						    operatorVotes[operatorsATK[i]], prettyATK,
							  operatorVotes[operatorsDEF[i]], prettyDEF);
				}

				fos.close ();
		} catch (e) {
				println ("Failed to write to '" + $.backseatSiege_VoteRankingsFile + "': " + e);
		}

    // $.backseatSiege_VoteRankingsFile
}

$.backseatSiege_setToggleState = function(state, sender)
{
    if (state == true )
    {
        $.backseatSiege_Toggle = true;
        $.inidb.set("settings", "backseatSiege_Toggle", "true");

        $.timer.addTimer("./systems/backseatSiegeSystem.js", "updateRankings", true, $.backseatSiege_updateRankings, (1 * 1000)); // update every 5 seconds

        if (sender != null)
            $.say($.lang.get("net.quorrabot.backseatSiegeSystem.voting-enabled"));
    }
    else
    {
        $.backseatSiege_Toggle = false;
        $.inidb.set("settings", "backseatSiege_Toggle", "false");

        $.timer.clearTimer("./systems/backseatSiegeSystem.js", "updateRankings", true);

        if (sender != null)
            $.say($.lang.get("net.quorrabot.backseatSiegeSystem.voting-disabled"));
    }
}

$.backseatSiege_resetVoting = function(sender, faction)
{
	  // Can reset voting for ATK, DEF and ALL.
		// Parameters have been verified by caller, so go with minimum working solution.

		var timestamp = $.backseatSiege_GetTime();

		if ($.backseatSiege_OperatorFactions[operator] != "def")   // ATK & ALL
		{
				$.backseatSiege_lastvote_atk = timestamp;
				$.inidb.set("backseatSiege_misc", "lastvote_atk", timestamp.toString());
		}
		if ($.backseatSiege_OperatorFactions[operator] != "atk")   // DEF & ALL
		{
				$.backseatSiege_lastvote_def = timestamp;
				$.inidb.set("backseatSiege_misc", "lastvote_def", timestamp.toString());
		}

	  $.say($.lang.get("net.quorrabot.backseatSiegeSystem.reset-done"));
}

$.backseatSiege_tamperWithVotes = function(sender, target, tamper)
{
	  var number = parseInt(tamper, 10);
	  var action = tamper.substring(0, 1);
		if ((action == "+") || (action == "-"))
		{
  			number = parseInt(tamper.substring(1), 10);
		}
		else
		{
	  		action = "=";
		}

		if (target in $.backseatSiege_OperatorFactions)
		{
	  		$.backseatSiege_tamperWithSpecificVote(sender, target, action, number);
		}
		else
		{
				// Implement.
				for (var operator in $.backseatSiege_OperatorFactions)
				{
						if ((target == $.backseatSiege_OperatorFactions[operator]) || (target == "all"))
						{
						    $.backseatSiege_tamperWithSpecificVote(sender, operator, action, number);
						}
				}
		}
}

$.backseatSiege_tamperWithSpecificVote = function(sender, operator, action, number)
{
    // Adjust operator's votes according to the given action and number.

    if (action == "=")
		{
			  // Absolute change. No need to query first.

			  backseatSiege_SetOperatorVoteCount(operator, number);
				return;
		}

    // Apparently we are to nudge the number to keep a semblance of democracy.

		var operator_votes = $.backseatSiege_GetOperatorVoteCount(operator);

		if (action == "+")
		{
			  operator_votes = operator_votes + number;
		}
		else if (action == "-")
		{
			operator_votes = operator_votes - number;
		}

	  backseatSiege_SetOperatorVoteCount(operator, operator_votes);
}

$.backseatSiege_GetTime = function()
{
	  // Javascript has no long type. But since we do most of our
	  // logic in Javascript, let's put it in a 'Number' which is
	  // hopefully accurate enough.
    return new Number(new java.util.Date().getTime().toString());
}

$.backseatSiege_CapitalizeFirstLetter = function(string) {
    return string.substring(0, 1).toUpperCase() + string.substring(1);
}

$.backseatSiege_GetOperatorVoteCount = function(operator) {
	  var operator_votes = parseInt($.inidb.get("backseatSiege_votes", operator));
	  if (operator_votes != operator_votes)
	  {
	  	  operator_votes = 0;
	  }
	  return operator_votes;
}

$.backseatSiege_SetOperatorVoteCount = function(operator, votes) {
	  $.inidb.set("backseatSiege_votes", operator, votes.toString());
}

$.backseatSiege_TallyVote = function(operator, sender, username)
{
	  // No error messages; only confirmation of successful tally.
	  // Too likely to end up as a spambot and get banned otherwise.

	  var factionTable = "";
	  var comparisonTime = "";
	  if ($.backseatSiege_OperatorFactions[operator] == "atk")
	  {
	      factionTable = "backseatSiege_lastvote_atk";
	      comparisonTime = $.backseatSiege_lastvote_atk;
	  }
	  if ($.backseatSiege_OperatorFactions[operator] == "def")
	  {
	      factionTable = "backseatSiege_lastvote_def";
	      comparisonTime = $.backseatSiege_lastvote_def;
	  }

	  // Check whether their last vote precedes streamers last choice for
	  // that particular operator faction.

	  var timestamp_lastvote = new Number($.inidb.get(factionTable, username));

	  if (timestamp_lastvote > comparisonTime)
	  {
	  	  // They already voted for this round. DENIED!
	  	  $.say($.getWhisperString(sender) + $.lang.get("net.quorrabot.backseatSiegeSystem.vote-spam-" + $.backseatSiege_OperatorFactions[operator], $.backseatSiege_CapitalizeFirstLetter(operator)));
	  	  return;
	  }

	  // Seems as if this vote is something we can work with.
	  // Register it and mark this users vote time.
	  var operator_votes = $.backseatSiege_GetOperatorVoteCount(operator);
	  operator_votes = operator_votes + 1;
	  backseatSiege_SetOperatorVoteCount(operator, operator_votes);

	  var timestamp_now = $.backseatSiege_GetTime();
	  $.inidb.set(factionTable, username, timestamp_now.toString());
}

$.backseatSiege_ConfirmChoice = function(operator, sender, username)
{
	  var votes = $.backseatSiege_GetOperatorVoteCount(operator);
		var vote_again_msg = $.lang.get("net.quorrabot.backseatSiegeSystem.vote-again-" + $.backseatSiege_OperatorFactions[operator]);
	  $.say($.lang.get("net.quorrabot.backseatSiegeSystem.operator-chosen", $.backseatSiege_CapitalizeFirstLetter(operator), votes) +
	        " " + vote_again_msg);

	  // Reset to votes to zero for that operator.
	  $.backseatSiege_SetOperatorVoteCount(operator, 0);

	  // Mark our last attacking/defending operator choice with a timestamp.
	  // This way, people can vote again for this faction when they want to.
	  var timestamp = $.backseatSiege_GetTime();
	  var factionKey = "";
	  if ($.backseatSiege_OperatorFactions[operator] == "atk")
	  {
	      factionKey = "lastvote_atk";
	      $.backseatSiege_lastvote_atk = timestamp;
	  }
	  if ($.backseatSiege_OperatorFactions[operator] == "def")
	  {
	      factionKey = "lastvote_def";
	      $.backseatSiege_lastvote_def = timestamp;
	  }

	  $.inidb.set("backseatSiege_misc", factionKey, timestamp.toString());

	  // Copy the image to match our needs.

	  // Copy over new image for use in stream.
	  var operator_file = backseatSiege_BadgesDirectory + operator + backseatSiege_BadgesExtension;
		backseatSiege_copyFile(operator_file, backseatSiege_BadgeRightNow);
}

$.on('command', function (event) {
    var sender = event.getSender().toLowerCase();
    var username = $.username.resolve(sender, event.getTags());
    var command = event.getCommand();
    var args = event.getArgs();

    if (command.equalsIgnoreCase("backseat"))
    {
				//if (!$.isAdmin(sender) || !$.isModv3(sender, event.getTags()))
				if (!$.isAdmin(sender))
				{
						$.say($.getWhisperString(sender) + $.modmsg);
						return;
				}

        if (args[0] == "toggle")
        {
            $.backseatSiege_setToggleState(!$.backseatSiege_Toggle, sender);
            return;
        }

        if (args[0] == "on")
        {
        	  $.backseatSiege_setToggleState(true, sender);
            return;
        }

        if (args[0] == "off")
        {
        	  $.backseatSiege_setToggleState(false, sender);
            return;
        }

				if (args[0] == "by")
				{
					  $.say("The backseatSiege voting plugin was originally written by @vitastic for his own use and has been opensourced at DonationCoder.com as a part of the NANY 2017 celebration.");
						return;
				}

				if (args[0] == "reset")
        {
					  if ((args[1] == "atK") || (args[1] == "def") || (args[1] == "all"))
						{
        	    $.backseatSiege_resetVoting(sender, args[1]);
              return;
					  }

						$.say($.lang.get("net.quorrabot.backseatSiegeSystem.reset-what"));
        }

				if (args[0] == "tamper")
				{
						// Valid operator ?
						var operator = args[1].toLowerCase();
						if ((operator in $.backseatSiege_OperatorFactions) ||
						    (operator == "all") || (operator == "atk") || (operator == "def"))
					  {
								// Valid tamper action ?
								var number = parseInt(args[2], 10);
								if (number == number)    // not NaN, thus a valid enough number for us.
								{
										$.backseatSiege_tamperWithVotes(sender, operator, args[2]);
										$.say($.lang.get("net.quorrabot.backseatSiegeSystem.tamper-complete"));
								}
								else
								{
										$.say($.lang.get("net.quorrabot.backseatSiegeSystem.tamper-invalid-tamper"));
								}
						}
						else
						{
								$.say($.lang.get("net.quorrabot.backseatSiegeSystem.tamper-invalid-target"));
						}


				}
    }
    else if (command.equalsIgnoreCase("play"))
    {
    	  if ($.backseatSiege_Toggle == false)
    	  {
    	  	  $.say($.getWhisperString(sender) + $.lang.get("net.quorrabot.backseatSiegeSystem.not-enabled"));
    	  	  return;
    	  }

    	  if (args[0] == null)
        {
            $.say($.getWhisperString(sender) + $.lang.get("net.quorrabot.backseatSiegeSystem.play-what"));
            return;
        }

    	  // Iterate through everything after the command.
    	  // People love their emotes and memes, after all.

    	  var argsLength = args.length;
    	  for (var i = 0; i < argsLength; i++)
    	  {
    	  	  var oneArg = args[i].toLowerCase();

            if (oneArg in $.backseatSiege_OperatorFactions)
            {
            	  // Found 'glaz', 'kapkan', etc.
            	  $.backseatSiege_TallyVote(oneArg, sender, username);
            }
        }
    }
    else if (command.equalsIgnoreCase("went"))
    {
			  // Allow mods to help streamer pick up the slack.
				if (!$.isAdmin(sender) || !$.isModv3(sender, event.getTags()))
				{
						$.say($.getWhisperString(sender) + $.modmsg);
						return;
				}

    	  if ($.backseatSiege_Toggle == false)
    	  {
    	  	  $.say($.getWhisperString(sender) + $.lang.get("net.quorrabot.backseatSiegeSystem.not-enabled"));
    	  	  return;
    	  }

    	  // Iterate through everything after the command.
    	  // People love their emotes and memes, after all.
    	  // And streamers are no different: !went tachanka like a boss!

    	  var argsLength = args.length;
    	  for (var i = 0; i < argsLength; i++)
    	  {
    	  	  var oneArg = args[i].toLowerCase();

            if (oneArg in $.backseatSiege_OperatorFactions)
            {
            	  // Found 'glaz', 'kapkan', etc.
            	  $.backseatSiege_ConfirmChoice(oneArg, sender, username);

            	  // One choice max. Return time!
            	  return;
            }
        }
    }
});

setTimeout(function () {

		if ($.moduleEnabled('./systems/backseatSiegeSystem.js')) {
	        backseatSiege_setToggleState($.backseatSiege_Toggle);
	  }

	  $.registerChatCommand("./systems/backseatSiegeSystem.js", "backseat");
	  $.registerChatCommand("./systems/backseatSiegeSystem.js", "play");
	  $.registerChatCommand("./systems/backseatSiegeSystem.js", "went");
}, 10 * 1000);

$.backseatSiege_InitializeDB = function()
{
    for (var operator in $.backseatSiege_OperatorFactions) {
        var v = $.backseatSiege_GetOperatorVoteCount(operator);

        if (v !== v)     // isNan
        {
       	    $.backseatSiege_SetOperatorVoteCount(operator, 0);
        }
		}

		if ($.backseatSiege_lastvote_atk.isNan())
		{
			  $.backseatSiege_lastvote_atk = $.backseatSiege_GetTime();
		}

		if ($.backseatSiege_lastvote_def.isNan())
		{
			  $.backseatSiege_lastvote_def = $.backseatSiege_GetTime();
		}
}
